---------------------
-- SEE LICENSE.TXT --
---------------------

if (not Watcher) then
    return;
end

---------------
-- LIBRARIES --
---------------
local L = LibStub("AceLocale-3.0"):GetLocale("Watcher");
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local MSQ = LibStub:GetLibrary("Masque", true);


----------------
-- MIGRATIONS --
----------------

function Watcher:Migrate()
    -- check for first launch
    if (not self.db.char.version) then
        self:FirstLaunch();
        return;
    end
    
    print ("Watcher is migrating from API version "..self.db.char.version.." to API version "..self.apiVersion..". This should convert older settings (including defaults) into the newer version and no action is required.");
    
    -- migrate from 0 to 1
    if (self.db.char.version == 0) then
        -- convert filters to newer version
        for filterSetId, filterSet in pairs(self.db.char.filterSets) do
            for filterId, filter in pairs(self.db.char.filterSets[filterSetId].filters) do
                if (filter.filterType == "auras") then
                    self:ConvertToNewAuraFilter(filter);
                elseif (filter.filterType == "health") then
                    self:ConvertToNewHealthFilter(filter);
                elseif (filter.filterType == "power") then
                    self:ConvertToNewPowerFilter(filter);
                elseif (filter.filterType == "talent") then
                    self:ConvertToNewTalentFilter(filter);
                elseif (filter.filterType == "spec") then
                    self:RemoveFilterSet(filterId);
                end
            end
        end
        
        self.db.char.version = 1;
    end
    
    -- migrate from 1 to 2
    if (self.db.char.version == 1) then
        -- convert all of the secondaryResource filters to use the new optionalBehavior
        for filterSetId, filterSet in pairs(self.db.char.filterSets) do
            for filterId, filter in pairs(self.db.char.filterSets[filterSetId].filters) do
                if (filter.filterType == "secondaryResource") then
                    self:ConvertToNewSecondaryResourceFilter(filter);
                end
            end
        end
    
        for priorityListId, priorityList in pairs(self.db.char.priorityLists) do
            -- add the cooldowns table
            self.db.char.priorityLists[priorityListId].cooldowns = {};
            
            -- remove spellCondition.name, since it doesn't need to exist.
            for spellConditionId, spellCondition in pairs(self.db.char.priorityLists[priorityListId].spellConditions) do
                if (self.db.char.priorityLists[priorityListId].spellConditions[spellConditionId].name) then
                    self.db.char.priorityLists[priorityListId].spellConditions[spellConditionId].name = nil;
                end
            end
        end
    
        self.db.char.version = 2;
    end
    
    self.db.char.version = self.apiVersion;
end

function Watcher:FirstLaunch()
    self.db.char.version = self.apiVersion;

    StaticPopupDialogs["Watcher First Launch"] = {
        text = L["first_launch"],
        button1 = L["Yes"],
        button2 = L["No"],
        OnAccept = function()
            InterfaceOptionsFrame_OpenToCategory("Watcher");
            InterfaceOptionsFrame_OpenToCategory("Watcher");
        end,
        timeout = 0,
        whileDead = true,
        hideOnEscape = false,
        preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
    };

    StaticPopup_Show ("Watcher First Launch");
end

function Watcher:ConvertToNewAuraFilter(filter)
    if (not filter.unit) then
        -- convert isBuff to auraType and unit
        if (filter.isBuff) then
            filter.auraType = "HELPFUL";
            filter.unit = "player";
        else
            filter.auraType = "HARMFUL";
            filter.unit = "target";
        end
        
        filter.isBuff = nil;
        
        -- convert stackCount to the various stacks options
        if (filter.stackCount == 0) then
            filter.trackStacks = false;
            filter.stackRelationship = "At Least";
        else
            filter.trackStacks = true;
            
            if (filter.invert) then
                filter.stackRelationship = "At Least";
            else
                filter.stackRelationship = "At Most";
            end
        end
        
        -- convert invert to ifExists
        if (filter.invert) then
            filter.ifExists = true;
        else
            filter.ifExists = false;
        end
        
        filter.invert = nil;
        
        -- convert refreshThreshold into trackRemainingTime
        if (filter.refreshThreshold == 0) then
            filter.trackRemainingTime = false;
            filter.remainingTimeRelationship = "BELOW";
            filter.remainingTime = 0;
        else
            filter.trackRemainingTime = true;
            filter.remainingTimeRelationship = "BELOW";
            filter.remainingTime = filter.refreshThreshold;
        end
        
        filter.refreshThreshold = nil;
    end
end

function Watcher:ConvertToNewHealthFilter(filter)
    if (not filter.relationship) then
        if (filter.player) then
            filter.unit = "player";
        else
            filter.unit = "target";
        end
        
        filter.player = nil;
        
        if (filter.invert) then
            filter.relationship = "ABOVE";
        else
            filter.relationship = "BELOW";
        end
        
        filter.invert = nil;
    end
end

function Watcher:ConvertToNewPowerFilter(filter)
    if (not filter.relationship) then
        if (filter.invert) then
            filter.relationship = "BELOW";
        else
            filter.relationship = "ABOVE";
        end
        
        filter.invert = nil;
    end
end

function Watcher:ConvertToNewTalentFilter(filter)
    if (filter.invert == nil) then
        filter.invert = false;
    end
end

function Watcher:ConvertToNewSecondaryResourceFilter(filter)
    if (not filter.optionalBehavior) then
        filter.optionalBehavior = "";
    end
end
